package main

import (
	"errors"
	"fmt"
	"github.com/mcuadros/go-version"
	"io/ioutil"
	"log"
	"os"
	"os/exec"
	"path/filepath"
	"strings"
)


// TODO: use wiki to ask users to set nuget config with `dotnet nuget add`

func ScanForDependencies(scanPath string) (allDeps []dependencyItem) {
	// Scan for dependencies
	files, err := ScanCsprojInDirectory(scanPath)
	logErrorIfAny(err, "ScanCsprojInDirectory")

	for _, file := range files {
		dependencies, err := ParseDependencies(file)
		logErrorIfAny(err, "ParseDependencies " + file)
		allDeps = append(allDeps, dependencies...)
	}
	return
}

func SyncPackages(nugetConfigPath, localRepoPath string, allDeps []dependencyItem) {
	log.Println("WARNING: We do not support custom nugetConfigPath yet. Please make sure your default nuget.config works. (usually ~/.nuget/NuGet/NuGet.Config)")

	// We must create a fake environment to make dotnet happy. Let's do that!
	// Create the fake environment directory.
	fakeProjectTemplate := "<Project Sdk=\"Microsoft.NET.Sdk\"><PropertyGroup><TargetFramework>%v</TargetFramework></PropertyGroup></Project>"
	fakeProjectDir, err := ioutil.TempDir("", "openxt-tmp")
	panicErrorIfAny(err, "createTempDir for sync package")
	defer os.RemoveAll(fakeProjectDir)

	// Prepare nuget local repo
	for _, dep := range allDeps {
		pkgName, targetNetVer := dep.pkgName, dep.targetNetVer
		if targetNetVer == "" {
			targetNetVer = "net472" // Default dotnet version
		}
		if DeductPkgNameFromVarName {
			guessedName := guessPkgNameFromVarName(dep.envName) // PkgCis_SdkV2 requires this to work.
			if guessedName != "" {
				pkgName = guessedName
			}
		}

		log.Println("DEBUG: Downloading pkg: " + pkgName + ":" + targetNetVer + " as var " + dep.envName)
		// Create the fake project.
		err = ioutil.WriteFile(filepath.Join(fakeProjectDir, "noob.csproj"), []byte(fmt.Sprintf(fakeProjectTemplate, targetNetVer)), 0666)
		logErrorIfAny(err, "write noob.csproj for " + pkgName)

		// dotnet add package Microsoft.Azure.Cis.Activities.Contracts --package-directory ~/tmp/your.repo # --framework is not working at all.
		cmd := exec.Command("dotnet", "add", "package", pkgName, "--package-directory", localRepoPath)
		cmd.Dir = fakeProjectDir
		stdout, err := cmd.Output()
		log.Println(string(stdout))
		// We can do nothing on failure. usually, if envName is NULL, we need not install it at all.
		logErrorIfAny(err, "EXEC command")
	}
}

func GetPackagePathFromName(localRepoPath, pkgName string) (pkgPath string, err error) {
	guessBase := localRepoPath + string(os.PathSeparator) + strings.ToLower(pkgName)
	files, err := ioutil.ReadDir(guessBase + string(os.PathSeparator))
	if err != nil {
		return
	}

	maxVersion := ""
	for _, f := range files {
		if f.IsDir() {
			log.Println("DEBUG: CURR_VER=" + f.Name())
			if version.CompareSimple(f.Name(), maxVersion) >= 0 {
				maxVersion = f.Name()
			}
		}
	}

	log.Println("DEBUG: MAX_VER=" + maxVersion)
	if maxVersion != "" {
		return guessBase + string(os.PathSeparator) + maxVersion, nil
	} else {
		return "", errors.New("No version found for package " + pkgName)
	}
}

